"use strict";

function _typescript() {
  var data = _interopRequireDefault(require("typescript"));

  _typescript = function _typescript() {
    return data;
  };

  return data;
}

function _path() {
  var data = _interopRequireDefault(require("path"));

  _path = function _path() {
    return data;
  };

  return data;
}

function _moduleDefinition() {
  var data = _interopRequireDefault(require("module-definition"));

  _moduleDefinition = function _moduleDefinition() {
    return data;
  };

  return data;
}

function _resolve() {
  var data = _interopRequireDefault(require("resolve"));

  _resolve = function _resolve() {
    return data;
  };

  return data;
}

function _moduleLookupAmd() {
  var data = _interopRequireDefault(require("module-lookup-amd"));

  _moduleLookupAmd = function _moduleLookupAmd() {
    return data;
  };

  return data;
}

function _stylusLookup() {
  var data = _interopRequireDefault(require("stylus-lookup"));

  _stylusLookup = function _stylusLookup() {
    return data;
  };

  return data;
}

function _sassLookup() {
  var data = _interopRequireDefault(require("sass-lookup"));

  _sassLookup = function _sassLookup() {
    return data;
  };

  return data;
}

function _resolveDependencyPath() {
  var data = _interopRequireDefault(require("resolve-dependency-path"));

  _resolveDependencyPath = function _resolveDependencyPath() {
    return data;
  };

  return data;
}

function _appModulePath() {
  var data = _interopRequireDefault(require("app-module-path"));

  _appModulePath = function _appModulePath() {
    return data;
  };

  return data;
}

function _enhancedResolve() {
  var data = _interopRequireDefault(require("enhanced-resolve"));

  _enhancedResolve = function _enhancedResolve() {
    return data;
  };

  return data;
}

function _isRelativePath() {
  var data = _interopRequireDefault(require("is-relative-path"));

  _isRelativePath = function _isRelativePath() {
    return data;
  };

  return data;
}

function _objectAssign() {
  var data = _interopRequireDefault(require("object-assign"));

  _objectAssign = function _objectAssign() {
    return data;
  };

  return data;
}

function _utils() {
  var data = require("../../utils");

  _utils = function _utils() {
    return data;
  };

  return data;
}

function _vueLookup() {
  var data = _interopRequireDefault(require("../lookups/vue-lookup"));

  _vueLookup = function _vueLookup() {
    return data;
  };

  return data;
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * this file had been forked from https://github.com/dependents/node-filing-cabinet
 */
var debug = require('debug')('cabinet');

var defaultLookups = {
  '.js': jsLookup,
  '.jsx': jsLookup,
  '.ts': tsLookup,
  '.tsx': tsLookup,
  '.scss': cssPreprocessorLookup,
  '.sass': cssPreprocessorLookup,
  '.styl': _stylusLookup().default,
  '.less': cssPreprocessorLookup,
  '.vue': _vueLookup().default
}; // for some reason, .ts is not sufficient, .d.ts is needed as well
// these extensions are used with commonJs and nonRelative lookups. When a partial doesn't have an
// extension it will look for files with these extensions in order.
// for example, `const a = require('.a')`, it'll look for a.js, a.jsx, a.ts and so on.

var resolveExtensions = Object.keys(defaultLookups).concat(['.d.ts', '.json', '.css']); // when webpack resolves dependencies from a style file, such as .scss file, look for style extensions only

var styleExtensions = ['.scss', '.sass', '.less', '.css'];

module.exports = function cabinet(options) {
  var partial = options.partial;
  var filename = options.filename;
  var directory = options.directory;

  var ext = options.ext || _path().default.extname(filename);

  debug(`working on a dependency "${partial}" of a file "${filename}"`);
  var resolver = defaultLookups[ext];

  if (!resolver) {
    debug('using generic resolver');
    resolver = _resolveDependencyPath().default;
  }

  if (ext === '.css' && partial.startsWith('~')) resolver = cssPreprocessorLookup;
  debug(`found a resolver ${resolver.name} for ${ext}`);

  var getResolverResults = function getResolverResults() {
    // old resolver are not getting an object parameter
    if (resolver.length > 1) {
      // check whether the 'resolver' function gets more than one parameter
      // $FlowFixMe
      return resolver(partial, filename, directory);
    }

    return resolver(options);
  };

  var result = getResolverResults();

  var partialExt = _path().default.extname(partial);

  if (!result && partialExt && partialExt !== ext) {
    resolver = defaultLookups[partialExt];

    if (resolver) {
      // todo: this strategy probably fails often. if, for example, a dependency A.js inside B.ts
      // was not found using the ts resolve, it tries the js resolver, however, parsing the ts file
      // with js parser, won't work most of the time. A better approach would be to fix the
      // original resolver.
      debug(`dependency has a different extension (${partialExt}) than the original file (${ext}), trying to use its resolver instead`);

      try {
        result = getResolverResults();
      } catch (err) {
        debug(`unable to use the resolver of ${partialExt} for ${filename}. got an error ${err.message}`);
      }
    }
  }

  debug(`resolved path for ${partial}: ${result}`);
  return result;
};

module.exports.supportedFileExtensions = Object.keys(defaultLookups);
/**
 * Register a custom lookup resolver for a file extension
 *
 * @param  {String} extension - The file extension that should use the resolver
 * @param  {Function} lookupStrategy - A resolver of partial paths
 */

module.exports.register = function (extension, lookupStrategy) {
  defaultLookups[extension] = lookupStrategy;

  if (this.supportedFileExtensions.indexOf(extension) === -1) {
    this.supportedFileExtensions.push(extension);
  }
};
/**
 * Exposed for testing
 *
 * @param  {Object} options
 * @param  {String} options.config
 * @param  {String} options.webpackConfig
 * @param  {String} options.filename
 * @param  {Object} options.ast
 * @return {String}
 */


module.exports._getJSType = function (options) {
  options = options || {};

  if (options.config) {
    return 'amd';
  }

  if (options.webpackConfig) {
    return 'webpack';
  }

  var ast = options.ast || options.content;

  if (ast) {
    debug('reusing the given ast or content');
    return _moduleDefinition().default.fromSource(ast);
  }

  debug('using the filename to find the module type');
  return _moduleDefinition().default.sync(options.filename);
};
/**
 * @private
 * @param  {String} partial
 * @param  {String} filename
 * @param  {String} directory
 * @param  {String} [config]
 * @param  {String} [webpackConfig]
 * @param  {String} [configPath]
 * @param  {Object} [ast]
 * @return {String}
 */


function jsLookup(options) {
  var configPath = options.configPath,
      partial = options.partial,
      directory = options.directory,
      config = options.config,
      webpackConfig = options.webpackConfig,
      filename = options.filename,
      ast = options.ast,
      isScript = options.isScript,
      content = options.content,
      resolveConfig = options.resolveConfig;

  var type = module.exports._getJSType({
    config,
    webpackConfig,
    filename,
    ast,
    isScript,
    content
  });

  switch (type) {
    case 'amd':
      debug('using amd resolver');
      return (0, _moduleLookupAmd().default)({
        config,
        // Optional in case a pre-parsed config is being passed in
        configPath,
        partial,
        directory,
        filename
      });

    case 'webpack':
      debug('using webpack resolver for es6');
      return resolveWebpackPath(partial, filename, directory, webpackConfig);

    case 'commonjs':
    case 'es6':
    default:
      debug(`using commonjs resolver ${type}`);
      return commonJSLookup(partial, filename, directory, resolveConfig);
  }
}
/**
 * from https://github.com/webpack-contrib/sass-loader
   webpack provides an [advanced mechanism to resolve files](https://webpack.js.org/concepts/module-resolution/).
   The sass-loader uses node-sass' custom importer feature to pass all queries to the webpack resolving engine.
   Thus you can import your Sass modules from `node_modules`. Just prepend them with a `~` to tell webpack that
   this is not a relative import:
    ```css
    @import "~bootstrap/dist/css/bootstrap";
      ```
    It's important to only prepend it with `~`, because `~/` resolves to the home directory.
    webpack needs to distinguish between `bootstrap` and `~bootstrap` because CSS and Sass files have no special
    syntax for importing relative files. Writing `@import "file"` is the same as `@import "./file";
 */


function cssPreprocessorLookup(options) {
  var filename = options.filename,
      partial = options.partial,
      directory = options.directory,
      resolveConfig = options.resolveConfig;

  if (resolveConfig && !(0, _utils().isRelativeImport)(partial)) {
    var result = resolveNonRelativePath(partial, filename, directory, resolveConfig);
    if (result) return result;
  }

  if (partial.startsWith('~') && !partial.startsWith('~/')) {
    // webpack syntax for resolving a module from node_modules
    debug('changing the resolver of css preprocessor to resolveWebpackPath as it has a ~ prefix');
    var partialWithNoTilda = partial.replace('~', '');
    return resolveWebpack(partialWithNoTilda, filename, directory, {
      extensions: styleExtensions,
      symlinks: false
    });
  } // Less and Sass imports are very similar


  return (0, _sassLookup().default)(partial, filename, directory);
}

function tsLookup(options) {
  var partial = options.partial,
      filename = options.filename,
      directory = options.directory,
      resolveConfig = options.resolveConfig;

  if (partial[0] !== '.') {
    // when a path is not relative, use the standard commonJS lookup
    return commonJSLookup(partial, filename, directory, resolveConfig);
  }

  debug('performing a typescript lookup');
  var tsOptions = {
    module: _typescript().default.ModuleKind.CommonJS
  };

  var host = _typescript().default.createCompilerHost({});

  debug('with options: ', tsOptions);

  var resolvedModule = _typescript().default.resolveModuleName(partial, filename, tsOptions, host).resolvedModule;

  if (!resolvedModule) {
    // for some reason, on Windows, ts.resolveModuleName method tries to find the module in the
    // root directory. For example, it searches for module './bar', in 'c:\bar.ts'.
    var fallbackModule = _path().default.resolve(_path().default.dirname(filename), partial);

    resolvedModule = _typescript().default.resolveModuleName(fallbackModule, filename, tsOptions, host).resolvedModule;
  }

  if (!resolvedModule) {
    // ts.resolveModuleName doesn't always work, fallback to commonJSLookup
    debug('failed resolving with tsLookup, trying commonJSLookup');
    return commonJSLookup(partial, filename, directory, resolveConfig);
  }

  debug('ts resolved module: ', resolvedModule);
  var result = resolvedModule ? resolvedModule.resolvedFileName : '';
  debug(`result: ${result}`);
  return result ? _path().default.resolve(result) : '';
}

function resolveNonRelativePath(partial, filename, directory, resolveConfig) {
  var webpackResolveConfig = {};
  if (resolveConfig.modulesDirectories) webpackResolveConfig.modules = resolveConfig.modulesDirectories;
  if (resolveConfig.aliases) webpackResolveConfig.alias = resolveConfig.aliases;
  webpackResolveConfig.extensions = resolveExtensions; // a resolve module might point to an imported component via the package name, in which case
  // the package name is a symlink to the imported component. we want it to be resolved as a pkg

  webpackResolveConfig.symlinks = false;
  return resolveWebpack(partial, filename, directory, webpackResolveConfig);
}
/**
 * TODO: Export to a separate module
 *
 * @private
 * @param  {String} partial
 * @param  {String} filename
 * @param  {String} directory
 * @return {String}
 */


function commonJSLookup(partial, filename, directory, resolveConfig) {
  directory = _path().default.dirname(filename); // node_modules should be propagated from the file location backwards
  // Need to resolve partials within the directory of the module, not filing-cabinet

  var moduleLookupDir = _path().default.join(directory, 'node_modules');

  debug(`adding ${moduleLookupDir} to the require resolution paths`);

  _appModulePath().default.addPath(moduleLookupDir); // Make sure the partial is being resolved to the filename's context
  // 3rd party modules will not be relative


  if (partial[0] === '.') {
    partial = _path().default.resolve(_path().default.dirname(filename), partial);
  }

  var result = '';

  try {
    result = _resolve().default.sync(partial, {
      extensions: resolveExtensions,
      basedir: directory,
      moduleDirectory: ['node_modules']
    });
    debug(`resolved path: ${result}`);
  } catch (e) {
    if (!(0, _utils().isRelativeImport)(partial) && resolveConfig) {
      debug(`trying to resolve using resolveConfig ${JSON.stringify(resolveConfig)}`);
      result = resolveNonRelativePath(partial, filename, directory, resolveConfig);
    } else {
      debug(`could not resolve ${partial}`);
    }
  }

  return result;
}

function resolveWebpackPath(partial, filename, directory, webpackConfig) {
  webpackConfig = _path().default.resolve(webpackConfig);

  try {
    var loadedConfig = require(webpackConfig);

    if (typeof loadedConfig === 'function') {
      loadedConfig = loadedConfig();
    }
  } catch (e) {
    debug(`error loading the webpack config at ${webpackConfig}`);
    debug(e.message);
    debug(e.stack);
    return '';
  }

  var resolveConfig = (0, _objectAssign().default)({}, loadedConfig.resolve);

  if (!resolveConfig.modules && (resolveConfig.root || resolveConfig.modulesDirectories)) {
    resolveConfig.modules = [];

    if (resolveConfig.root) {
      resolveConfig.modules = resolveConfig.modules.concat(resolveConfig.root);
    }

    if (resolveConfig.modulesDirectories) {
      resolveConfig.modules = resolveConfig.modules.concat(resolveConfig.modulesDirectories);
    }
  }

  return resolveWebpack(partial, filename, directory, resolveConfig);
}

function resolveWebpack(partial, filename, directory, resolveConfig) {
  try {
    var resolver = _enhancedResolve().default.create.sync(resolveConfig); // We don't care about what the loader resolves the partial to
    // we only want the path of the resolved file


    partial = stripLoader(partial);
    var lookupPath = (0, _isRelativePath().default)(partial) ? _path().default.dirname(filename) : directory;
    return resolver(lookupPath, partial);
  } catch (e) {
    debug(`error when resolving ${partial}`);
    debug(e.message);
    debug(e.stack);
    return '';
  }
}

function stripLoader(partial) {
  var exclamationLocation = partial.indexOf('!');

  if (exclamationLocation === -1) {
    return partial;
  }

  return partial.slice(exclamationLocation + 1);
}
//# sourceMappingURL=index.js.map